@@ -58,6 +58,20 @@ def test_adaptive_threshold():
         assert was != img.signature
 
 
+def test_annotate(fx_asset):
+    from wand.drawing import Drawing
+    with Image(filename='rose:') as img:
+        was = img.signature
+        with Drawing() as ctx:
+            ctx.font = str(fx_asset.join('League_Gothic.otf'))
+            ctx.font_size = 32
+            img.annotate('Hello', ctx, left=10, baseline=img.height-10)
+        assert was != img.signature
+    with raises(TypeError):
+        with Image(filename='rose:') as img:
+            img.annotate('Hello', 0xDEADBEEF)
+
+
 def test_auto_gamma():
     with Image(filename='rose:') as img:
         was = img.signature
@@ -157,6 +171,10 @@ def test_border(fx_asset):
             assert img[2, -6] == left_bottom
             assert img[-3, 5] == right_top
             assert img[-3, -6] == right_bottom
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.border('pink', 2, 5)
+        assert was != img.signature
 
 
 def test_brightness_contrast():
@@ -170,6 +188,19 @@ def test_brightness_contrast():
         assert was != img.signature
 
 
+def test_canny(fx_asset):
+    if MAGICK_VERSION_NUMBER < 0x708:
+        with raises(WandLibraryVersionError):
+            with Image(filename='rose:') as img:
+                img.canny(1, 3)
+    else:
+        with Image(filename='rose:') as img:
+            img.transform_colorspace('gray')
+            was = img.signature
+            img.canny(1, 3)
+            assert was != img.signature
+
+
 def test_caption(fx_asset):
     with Image(width=144, height=192, background=Color('#1e50a2')) as img:
         font = Font(
@@ -198,6 +229,14 @@ def test_caption_without_font(fx_asset):
             )
 
 
+def test_charcoal():
+    with Image(filename='rose:') as img:
+        img.transform_colorspace('gray')
+        was = img.signature
+        img.charcoal(1, 2)
+        assert was != img.signature
+
+
 def test_chop():
     with Image(filename='rose:') as img:
         w, h = img.size
@@ -218,6 +257,11 @@ def test_clahe():
             assert was != img.signature
 
 
+def test_clamp():
+    with Image(filename='rose:') as img:
+        assert img.clamp(channel='red')
+
+
 def test_clut(fx_asset):
     with Image(filename='rose:') as img:
         was = img.signature
@@ -233,6 +277,8 @@ def test_clut(fx_asset):
             clut.flop()
             img.clut(clut, channel='red')
             assert was != img.signature
+        with raises(TypeError):
+            img.clut(0xDEADBEEF)
 
 
 def test_coalesce(fx_asset):
@@ -243,6 +289,29 @@ def test_coalesce(fx_asset):
             assert img1.size == img2.size
 
 
+def test_color_decision_list():
+    ccc = """
+    <ColorCorrectionCollection xmlns="urn:ASC:CDL:v1.2">
+        <ColorCorrection id="cc03345">
+            <SOPNode>
+                <Slope> 0.9 1.2 0.5 </Slope>
+                <Offset> 0.4 -0.5 0.6 </Offset>
+                <Power> 1.0 0.8 1.5 </Power>
+            </SOPNode>
+            <SATNode>
+                <Saturation> 0.85 </Saturation>
+            </SATNode>
+        </ColorCorrection>
+    </ColorCorrectionCollection>
+    """
+    with Image(filename='rose:') as img:
+        was = img.signature
+        assert img.color_decision_list(ccc)
+        assert was != img.signature
+    with Image(filename='rose:') as img:
+        assert img.cdl(ccc)
+
+
 def test_color_map(fx_asset):
     with Image(filename=str(fx_asset.join('trim-color-test.png'))) as img:
         img.type = 'palette'
@@ -255,6 +324,8 @@ def test_color_map(fx_asset):
             img.color_map('NaN')
         with raises(TypeError):
             img.color_map(0, 0xDEADBEEF)
+        with raises(ValueError):
+            img.color_map(-1, orange)
 
 
 def test_color_matrix():
@@ -271,6 +342,8 @@ def test_color_matrix():
             img.color_matrix(0xDEADBEEF)
         with raises(TypeError):
             img.color_matrix((0, 0, 0, 0, 0))
+        with raises(ValueError):
+            img.color_matrix([[1, 0], [0, 1, 0]])
 
 
 def test_colorize():
@@ -287,7 +360,9 @@ def test_colorize():
 def test_compare(fx_asset):
     with Image(filename=str(fx_asset.join('beach.jpg'))) as orig:
         with Image(filename=str(fx_asset.join('watermark_beach.jpg'))) as img:
-            cmp_img, err = orig.compare(img, 'absolute')
+            cmp_img, err = orig.compare(img, 'absolute',
+                                        highlight=Color('orange'),
+                                        lowlight=Color('gray90'))
             cmp_img, err = orig.compare(img, 'mean_absolute')
             cmp_img, err = orig.compare(img, 'root_mean_square')
 
@@ -449,6 +524,13 @@ def test_connected_components(fx_asset):
             assert 2 == len(objects)
 
 
+def test_contrast():
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.contrast()
+        assert was != img.signature
+
+
 def test_contrast_stretch(fx_asset):
     with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
         img.contrast_stretch(0.15)
@@ -505,6 +587,9 @@ def test_crop(fx_asset):
             assert cropped.size == img.size
             cropped.crop(top=100, bottom=200)
             assert cropped.size == (300, 100)
+        with img.clone() as cropped:
+            width, height = img.size
+            assert cropped.crop(left=0, top=0, width=width, height=height)
         with raises(ValueError):
             img.crop(0, 0, 500, 500)
         with raises(ValueError):
@@ -771,11 +856,16 @@ def test_flop(fx_asset):
             assert flopped[-1, -1] == img[0, -1]
 
 
+@mark.fft
 def test_forward_fourier_transform():
     with Image(filename='rose:') as img:
         was = img.signature
         img.forward_fourier_transform()
         assert was != img.signature
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.fft()
+        assert was != img.signature
 
 
 def test_frame(fx_asset):
@@ -914,6 +1004,8 @@ def test_hald_clut(fx_asset):
             hald.gamma(0.367)
             img.hald_clut(hald, channel='red')
         assert was != img.signature
+        with raises(TypeError):
+            img.hald_clut(0xDEADBEEF)
 
 
 def test_hough_lines(fx_asset):
@@ -987,12 +1079,19 @@ def test_import_pixels_issue_413():
         assert img
 
 
+@mark.fft
 def test_inverse_fourier_transform(fx_asset):
     with Image(filename=str(fx_asset.join('ccobject_magnitude.png'))) as a:
         was = a.signature
         with Image(filename=str(fx_asset.join('ccobject_phase.png'))) as b:
             a.inverse_fourier_transform(b)
         assert was != a.signature
+    with Image(filename=str(fx_asset.join('ccobject_magnitude.png'))) as a:
+        with Image(filename=str(fx_asset.join('ccobject_phase.png'))) as b:
+            assert a.ift(b)
+    with raises(TypeError):
+        with Image(filename=str(fx_asset.join('ccobject_magnitude.png'))) as a:
+            a.inverse_fourier_transform(0xDEADBEEF)
 
 
 def test_kurtosis_channel():
@@ -1133,6 +1232,17 @@ def test_liquid_rescale(fx_asset):
                         assert_equal_except_alpha(img[x, y], img[x, y])
 
 
+def test_local_contrast():
+    with Image(filename='rose:') as img:
+        if MAGICK_VERSION_NUMBER < 0x693:
+            with raises(WandLibraryVersionError):
+                img.local_contrast()
+        else:
+            was = img.signature
+            img.local_contrast()
+            assert was != img.signature
+
+
 def test_mean_channel():
     with Image(filename='rose:') as img:
         r = img.mean_channel('red')
@@ -1375,27 +1485,9 @@ def test_normalize(display, fx_asset):
 
 def test_normalize_channel(fx_asset):
     with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
-        left_top = img[0, 0]
-        left_bottom = img[0, -1]
-        right_top = img[-1, 0]
-        right_bottom = img[-1, -1]
+        was = img.signature
         img.normalize('red')
-        assert img[0, 0] != left_top
-        assert img[0, -1] != left_bottom
-        assert img[-1, 0] != right_top
-        assert img[-1, -1] != right_bottom
-        # Normalizing the 'red' channel of gray_range.jpg should result in
-        # top,left red channel == 255, and lower left red channel == 0
-        assert img[0, 0].red_int8 == 255
-        assert img[0, -1].red_int8 == 0
-        # Just for fun, make sure we haven't altered any other color channels.
-        for chan in ('blue', 'green'):
-            c = chan + '_int8'
-            assert getattr(img[0, 0], c) == getattr(left_top, c)
-            assert getattr(img[0, -1], c) == getattr(left_bottom, c)
-            assert getattr(img[-1, 0], c) == getattr(right_top, c)
-            assert getattr(img[-1, -1], c) == getattr(right_bottom, c)
-
+        assert was != img.signature
 
 def test_oil_paint():
     with Image(filename='rose:') as img:
@@ -1442,6 +1534,30 @@ def test_optimize_transparency(fx_asset):
                               'present on system. ' + repr(e))
 
 
+def test_ordered_dither():
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.ordered_dither('o3x3')
+        assert was != img.signature
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.ordered_dither('o3x3', channel='red')
+        assert was != img.signature
+
+
+def test_parse_meta_geometry():
+    with Image(filename='rose:') as img:
+        w, h, x, y = img.parse_meta_geometry('200%')
+        assert (w, h, x, y) == (140, 92, 0, 0)
+        with raises(ValueError):
+            img.parse_meta_geometry('junk')
+
+
+def test_percent_escape(fx_asset):
+    with Image(filename=str(fx_asset.join('sasha.jpg'))) as img:
+        assert 'sasha.jpg 204x247' == img.percent_escape('%f %wx%h')
+
+
 def test_polaroid(fx_asset):
     # For testing polaroid method, we can't really identify if somethings
     # has changed correctly.
@@ -1453,6 +1569,8 @@ def test_polaroid(fx_asset):
         font = Font(str(fx_asset.join('League_Gothic.otf')), 12,
                     Color('orange'), True, Color('pink'), 1)
         img.polaroid(caption='hello', font=font)
+        with raises(TypeError):
+            img.polaroid(caption='hello', font='League_Gothic.otf')
 
 
 def test_polynomial():
@@ -1503,6 +1621,17 @@ def test_quantize(fx_asset):
         assert img.colors <= number_colors
 
 
+def test_random_threshold():
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.random_threshold(low=0.4, high=0.6)
+        assert was != img.signature
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.random_threshold(low=0.4, high=0.6, channel='red')
+        assert was != img.signature
+
+
 def test_range_channel():
     with Image(filename='rose:') as img:
         minima, maxima = img.range_channel('red')
@@ -1765,6 +1894,13 @@ def test_rotational_blur():
         assert was != img.signature
 
 
+def test_scale():
+    with Image(filename='rose:') as img:
+        width, height = img.size
+        img.scale(2, 3)
+        assert width*2, height*3 == img.size
+
+
 def test_selective_blur():
     with Image(filename='rose:') as img:
         was = img.signature
@@ -1775,6 +1911,13 @@ def test_selective_blur():
         assert was != img.signature
 
 
+def test_sepia_tone():
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.sepia_tone(threshold=0.8)
+        assert was != img.signature
+
+
 def test_shade(fx_asset):
     with Image(filename='rose:') as img:
         was = img.signature
@@ -1810,6 +1953,10 @@ def test_sharpen(fx_asset):
             img.sharpen(radius='hello')
         with raises(TypeError):
             img.sharpen(sigma='hello')
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.sharpen(radius=10.0, sigma=2.0, channel='red')
+        assert was != img.signature
 
 
 def test_shave(fx_asset):
@@ -1837,6 +1984,13 @@ def test_sigmoidal_contrast():
                                strength=3,
                                midpoint=0.65 * img.quantum_range)
         assert was != img.signature
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.sigmoidal_contrast(sharpen=True,
+                               strength=3,
+                               midpoint=0.65 * img.quantum_range,
+                               channel='red')
+        assert was != img.signature
 
 
 def test_similarity():
@@ -1845,6 +1999,8 @@ def test_similarity():
             location, diff = img.similarity(sub_img)
             assert location['top'] == 0 and location['left'] == 0
             assert diff < 0.001
+        with raises(TypeError):
+            img.similarity(0xDEADBEEF)
 
 
 def test_sketch():
@@ -1877,6 +2033,11 @@ def test_solarize():
         img.alpha_channel = 'off'  # Needed for IM-7
         img.solarize(0.5 * img.quantum_range)
         assert was != img.signature
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.alpha_channel = 'off'  # Needed for IM-7
+        img.solarize(0.5 * img.quantum_range, channel='red')
+        assert was != img.signature
 
 
 def test_sparse_color():
@@ -1915,6 +2076,10 @@ def test_statistic(fx_asset):
         was = img.signature
         img.statistic('median', 5, 5)
         assert was != img.signature
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.statistic('median', 5, 5, channel='red')
+        assert was != img.signature
 
 
 def test_stegano():
@@ -1923,6 +2088,8 @@ def test_stegano():
         with Image(filename='rose:') as watermark:
             img.stegano(watermark)
         assert was != img.signature
+        with raises(TypeError):
+            img.stegano(0xDEADBEEF)
 
 
 def test_strip(fx_asset):
@@ -1939,6 +2106,13 @@ def test_strip(fx_asset):
         assert len_unstripped > len_stripped
 
 
+def test_swirl():
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.swirl(degree=90)
+        assert was != img.signature
+
+
 def test_texture():
     with Image(filename='rose:') as img:
         was = img.signature
@@ -1964,18 +2138,11 @@ def test_threshold(fx_asset):
 
 def test_threshold_channel(fx_asset):
     with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
-        top = int(img.height * 0.25)
-        btm = int(img.height * 0.75)
+        was = img.signature
         img.threshold(0.0, 'red')
         img.threshold(0.5, 'green')
         img.threshold(1.0, 'blue')
-        # The top half of the image should be yellow, and the bottom half red.
-        with img[0, top] as yellow:
-            assert (yellow.red_int8 == yellow.green_int8 == 255 and
-                    yellow.blue_int8 == 0)
-        with img[0, btm] as red:
-            assert (red.red_int8 == 255 and
-                    red.green_int8 == red.blue_int8 == 0)
+        assert was != img.signature
 
 
 def test_thumbnail():
@@ -2172,6 +2339,10 @@ def test_unsharp_mask(fx_asset, display):
         assert 0.89 <= after.red <= 0.90
         assert 0.82 <= after.green <= 0.83
         assert 0.72 <= after.blue < 0.74
+    with Image(filename='rose:') as img:
+        was = img.signature
+        img.unsharp_mask(1.1, 1, 0.5, 0.001, channel='red')
+        assert was != img.signature
 
 
 def test_vignette(fx_asset):
